@@ -108,9 +108,9 @@ t_int *scalarplus_perf8(t_int *w)
 void dsp_add_plus(t_sample *in1, t_sample *in2, t_sample *out, int n)
 {
     if (n&7)
-        dsp_add(plus_perform, 4, in1, in2, out, n);
+        dsp_add(plus_perform, 4, in1, in2, out, (t_int)n);
     else
-        dsp_add(plus_perf8, 4, in1, in2, out, n);
+        dsp_add(plus_perf8, 4, in1, in2, out, (t_int)n);
 }
 
 static void plus_dsp(t_plus *x, t_signal **sp)
@@ -122,10 +122,10 @@ static void scalarplus_dsp(t_scalarplus *x, t_signal **sp)
 {
     if (sp[0]->s_n&7)
         dsp_add(scalarplus_perform, 4, sp[0]->s_vec, &x->x_g,
-            sp[1]->s_vec, sp[0]->s_n);
+            sp[1]->s_vec, (t_int)sp[0]->s_n);
     else
         dsp_add(scalarplus_perf8, 4, sp[0]->s_vec, &x->x_g,
-            sp[1]->s_vec, sp[0]->s_n);
+            sp[1]->s_vec, (t_int)sp[0]->s_n);
 }
 
 static void plus_setup(void)
@@ -242,20 +242,20 @@ static void minus_dsp(t_minus *x, t_signal **sp)
 {
     if (sp[0]->s_n&7)
         dsp_add(minus_perform, 4,
-            sp[0]->s_vec, sp[1]->s_vec, sp[2]->s_vec, sp[0]->s_n);
+            sp[0]->s_vec, sp[1]->s_vec, sp[2]->s_vec, (t_int)sp[0]->s_n);
     else
         dsp_add(minus_perf8, 4,
-            sp[0]->s_vec, sp[1]->s_vec, sp[2]->s_vec, sp[0]->s_n);
+            sp[0]->s_vec, sp[1]->s_vec, sp[2]->s_vec, (t_int)sp[0]->s_n);
 }
 
 static void scalarminus_dsp(t_scalarminus *x, t_signal **sp)
 {
     if (sp[0]->s_n&7)
         dsp_add(scalarminus_perform, 4, sp[0]->s_vec, &x->x_g,
-            sp[1]->s_vec, sp[0]->s_n);
+            sp[1]->s_vec, (t_int)sp[0]->s_n);
     else
         dsp_add(scalarminus_perf8, 4, sp[0]->s_vec, &x->x_g,
-            sp[1]->s_vec, sp[0]->s_n);
+            sp[1]->s_vec, (t_int)sp[0]->s_n);
 }
 
 static void minus_setup(void)
@@ -373,20 +373,20 @@ static void times_dsp(t_times *x, t_signal **sp)
 {
     if (sp[0]->s_n&7)
         dsp_add(times_perform, 4,
-            sp[0]->s_vec, sp[1]->s_vec, sp[2]->s_vec, sp[0]->s_n);
+            sp[0]->s_vec, sp[1]->s_vec, sp[2]->s_vec, (t_int)sp[0]->s_n);
     else
         dsp_add(times_perf8, 4,
-            sp[0]->s_vec, sp[1]->s_vec, sp[2]->s_vec, sp[0]->s_n);
+            sp[0]->s_vec, sp[1]->s_vec, sp[2]->s_vec, (t_int)sp[0]->s_n);
 }
 
 static void scalartimes_dsp(t_scalartimes *x, t_signal **sp)
 {
     if (sp[0]->s_n&7)
         dsp_add(scalartimes_perform, 4, sp[0]->s_vec, &x->x_g,
-            sp[1]->s_vec, sp[0]->s_n);
+            sp[1]->s_vec, (t_int)sp[0]->s_n);
     else
         dsp_add(scalartimes_perf8, 4, sp[0]->s_vec, &x->x_g,
-            sp[1]->s_vec, sp[0]->s_n);
+            sp[1]->s_vec, (t_int)sp[0]->s_n);
 }
 
 static void times_setup(void)
@@ -515,20 +515,20 @@ static void over_dsp(t_over *x, t_signal **sp)
 {
     if (sp[0]->s_n&7)
         dsp_add(over_perform, 4,
-            sp[0]->s_vec, sp[1]->s_vec, sp[2]->s_vec, sp[0]->s_n);
+            sp[0]->s_vec, sp[1]->s_vec, sp[2]->s_vec, (t_int)sp[0]->s_n);
     else
         dsp_add(over_perf8, 4,
-            sp[0]->s_vec, sp[1]->s_vec, sp[2]->s_vec, sp[0]->s_n);
+            sp[0]->s_vec, sp[1]->s_vec, sp[2]->s_vec, (t_int)sp[0]->s_n);
 }
 
 static void scalarover_dsp(t_scalarover *x, t_signal **sp)
 {
     if (sp[0]->s_n&7)
         dsp_add(scalarover_perform, 4, sp[0]->s_vec, &x->x_g,
-            sp[1]->s_vec, sp[0]->s_n);
+            sp[1]->s_vec, (t_int)sp[0]->s_n);
     else
         dsp_add(scalarover_perf8, 4, sp[0]->s_vec, &x->x_g,
-            sp[1]->s_vec, sp[0]->s_n);
+            sp[1]->s_vec, (t_int)sp[0]->s_n);
 }
 
 static void over_setup(void)
@@ -657,20 +657,20 @@ static void max_dsp(t_max *x, t_signal **sp)
 {
     if (sp[0]->s_n&7)
         dsp_add(max_perform, 4,
-            sp[0]->s_vec, sp[1]->s_vec, sp[2]->s_vec, sp[0]->s_n);
+            sp[0]->s_vec, sp[1]->s_vec, sp[2]->s_vec, (t_int)sp[0]->s_n);
     else
         dsp_add(max_perf8, 4,
-            sp[0]->s_vec, sp[1]->s_vec, sp[2]->s_vec, sp[0]->s_n);
+            sp[0]->s_vec, sp[1]->s_vec, sp[2]->s_vec, (t_int)sp[0]->s_n);
 }
 
 static void scalarmax_dsp(t_scalarmax *x, t_signal **sp)
 {
     if (sp[0]->s_n&7)
         dsp_add(scalarmax_perform, 4, sp[0]->s_vec, &x->x_g,
-            sp[1]->s_vec, sp[0]->s_n);
+            sp[1]->s_vec, (t_int)sp[0]->s_n);
     else
         dsp_add(scalarmax_perf8, 4, sp[0]->s_vec, &x->x_g,
-            sp[1]->s_vec, sp[0]->s_n);
+            sp[1]->s_vec, (t_int)sp[0]->s_n);
 }
 
 static void max_setup(void)
@@ -780,7 +780,7 @@ t_int *scalarmin_perf8(t_int *w)
 {
     t_sample *in = (t_sample *)(w[1]);
     t_float g = *(t_float *)(w[2]);
-    t_float *out = (t_float *)(w[3]);
+    t_sample *out = (t_sample *)(w[3]);
     int n = (int)(w[4]);
     for (; n; n -= 8, in += 8, out += 8)
     {
@@ -799,20 +799,20 @@ static void min_dsp(t_min *x, t_signal **sp)
 {
     if (sp[0]->s_n&7)
         dsp_add(min_perform, 4,
-            sp[0]->s_vec, sp[1]->s_vec, sp[2]->s_vec, sp[0]->s_n);
+            sp[0]->s_vec, sp[1]->s_vec, sp[2]->s_vec, (t_int)sp[0]->s_n);
     else
         dsp_add(min_perf8, 4,
-            sp[0]->s_vec, sp[1]->s_vec, sp[2]->s_vec, sp[0]->s_n);
+            sp[0]->s_vec, sp[1]->s_vec, sp[2]->s_vec, (t_int)sp[0]->s_n);
 }
 
 static void scalarmin_dsp(t_scalarmin *x, t_signal **sp)
 {
     if (sp[0]->s_n&7)
         dsp_add(scalarmin_perform, 4, sp[0]->s_vec, &x->x_g,
-            sp[1]->s_vec, sp[0]->s_n);
+            sp[1]->s_vec, (t_int)sp[0]->s_n);
     else
         dsp_add(scalarmin_perf8, 4, sp[0]->s_vec, &x->x_g,
-            sp[1]->s_vec, sp[0]->s_n);
+            sp[1]->s_vec, (t_int)sp[0]->s_n);
 }
 
 static void min_setup(void)
