@@ -96,7 +96,7 @@ static void tabwrite_tilde_set(t_tabwrite_tilde *x, t_symbol *s)
 static void tabwrite_tilde_dsp(t_tabwrite_tilde *x, t_signal **sp)
 {
     tabwrite_tilde_set(x, x->x_arrayname);
-    dsp_add(tabwrite_tilde_perform, 3, x, sp[0]->s_vec, sp[0]->s_n);
+    dsp_add(tabwrite_tilde_perform, 3, x, sp[0]->s_vec, (t_int)sp[0]->s_n);
 }
 
 static void tabwrite_tilde_bang(t_tabwrite_tilde *x)
@@ -221,7 +221,7 @@ static void tabplay_tilde_set(t_tabplay_tilde *x, t_symbol *s)
 static void tabplay_tilde_dsp(t_tabplay_tilde *x, t_signal **sp)
 {
     tabplay_tilde_set(x, x->x_arrayname);
-    dsp_add(tabplay_tilde_perform, 3, x, sp[0]->s_vec, sp[0]->s_n);
+    dsp_add(tabplay_tilde_perform, 3, x, sp[0]->s_vec, (t_int)sp[0]->s_n);
 }
 
 static void tabplay_tilde_list(t_tabplay_tilde *x, t_symbol *s,
@@ -343,7 +343,7 @@ static void tabread_tilde_dsp(t_tabread_tilde *x, t_signal **sp)
     tabread_tilde_set(x, x->x_arrayname);
 
     dsp_add(tabread_tilde_perform, 4, x,
-        sp[0]->s_vec, sp[1]->s_vec, sp[0]->s_n);
+        sp[0]->s_vec, sp[1]->s_vec, (t_int)sp[0]->s_n);
 
 }
 
@@ -474,7 +474,7 @@ static void tabread4_tilde_dsp(t_tabread4_tilde *x, t_signal **sp)
     tabread4_tilde_set(x, x->x_arrayname);
 
     dsp_add(tabread4_tilde_perform, 4, x,
-        sp[0]->s_vec, sp[1]->s_vec, sp[0]->s_n);
+        sp[0]->s_vec, sp[1]->s_vec, (t_int)sp[0]->s_n);
 
 }
 
@@ -659,7 +659,7 @@ static void tabosc4_tilde_dsp(t_tabosc4_tilde *x, t_signal **sp)
     tabosc4_tilde_set(x, x->x_arrayname);
 
     dsp_add(tabosc4_tilde_perform, 4, x,
-        sp[0]->s_vec, sp[1]->s_vec, sp[0]->s_n);
+        sp[0]->s_vec, sp[1]->s_vec, (t_int)sp[0]->s_n);
 }
 
 static void tabosc4_tilde_setup(void)
@@ -759,7 +759,7 @@ static void tabsend_dsp(t_tabsend *x, t_signal **sp)
     if (ticksper < 1) ticksper = 1;
     x->x_graphperiod = ticksper;
     if (x->x_graphcount > ticksper) x->x_graphcount = ticksper;
-    dsp_add(tabsend_perform, 3, x, sp[0]->s_vec, n);
+    dsp_add(tabsend_perform, 3, x, sp[0]->s_vec, (t_int)n);
 }
 
 static void tabsend_setup(void)
@@ -831,7 +831,7 @@ static void tabreceive_set(t_tabreceive *x, t_symbol *s)
 static void tabreceive_dsp(t_tabreceive *x, t_signal **sp)
 {
     tabreceive_set(x, x->x_arrayname);
-    dsp_add(tabreceive_perform, 3, x, sp[0]->s_vec, sp[0]->s_n);
+    dsp_add(tabreceive_perform, 3, x, sp[0]->s_vec, (t_int)sp[0]->s_n);
 }
 
 static void *tabreceive_new(t_symbol *s)
