@@ -107,18 +107,18 @@ static void sigfft_dspx(t_sigfft *x, t_signal **sp, t_int *(*f)(t_int *w))
     t_sample *out1 = sp[2]->s_vec;
     t_sample *out2 = sp[3]->s_vec;
     if (out1 == in2 && out2 == in1)
-        dsp_add(sigfft_swap, 3, out1, out2, n);
+        dsp_add(sigfft_swap, 3, out1, out2, (t_int)n);
     else if (out1 == in2)
     {
-        dsp_add(copy_perform, 3, in2, out2, n);
-        dsp_add(copy_perform, 3, in1, out1, n);
+        dsp_add(copy_perform, 3, in2, out2, (t_int)n);
+        dsp_add(copy_perform, 3, in1, out1, (t_int)n);
     }
     else
     {
-        if (out1 != in1) dsp_add(copy_perform, 3, in1, out1, n);
-        if (out2 != in2) dsp_add(copy_perform, 3, in2, out2, n);
+        if (out1 != in1) dsp_add(copy_perform, 3, in1, out1, (t_int)n);
+        if (out2 != in2) dsp_add(copy_perform, 3, in2, out2, (t_int)n);
     }
-    dsp_add(f, 3, sp[2]->s_vec, sp[3]->s_vec, n);
+    dsp_add(f, 3, sp[2]->s_vec, sp[3]->s_vec, (t_int)n);
 }
 
 static void sigfft_dsp(t_sigfft *x, t_signal **sp)
@@ -190,9 +190,9 @@ static void sigrfft_dsp(t_sigrfft *x, t_signal **sp)
         return;
     }
     if (in1 != out1)
-        dsp_add(copy_perform, 3, in1, out1, n);
-    dsp_add(sigrfft_perform, 2, out1, n);
-    dsp_add(sigrfft_flip, 3, out1 + (n2+1), out2 + n2, n2-1);
+        dsp_add(copy_perform, 3, in1, out1, (t_int)n);
+    dsp_add(sigrfft_perform, 2, out1, (t_int)n);
+    dsp_add(sigrfft_flip, 3, out1 + (n2+1), out2 + n2, (t_int)(n2-1));
     dsp_add_zero(out1 + (n2+1), ((n2-1)&(~7)));
     dsp_add_zero(out1 + (n2+1) + ((n2-1)&(~7)), ((n2-1)&7));
     dsp_add_zero(out2 + n2, n2);
@@ -251,15 +251,15 @@ static void sigrifft_dsp(t_sigrifft *x, t_signal **sp)
     }
     if (in2 == out1)
     {
-        dsp_add(sigrfft_flip, 3, out1+1, out1 + n, n2-1);
-        dsp_add(copy_perform, 3, in1, out1, n2+1);
+        dsp_add(sigrfft_flip, 3, out1+1, out1 + n, (t_int)(n2-1));
+        dsp_add(copy_perform, 3, in1, out1, (t_int)(n2+1));
     }
     else
     {
-        if (in1 != out1) dsp_add(copy_perform, 3, in1, out1, n2+1);
-        dsp_add(sigrfft_flip, 3, in2+1, out1 + n, n2-1);
+        if (in1 != out1) dsp_add(copy_perform, 3, in1, out1, (t_int)(n2+1));
+        dsp_add(sigrfft_flip, 3, in2+1, out1 + n, (t_int)(n2-1));
     }
-    dsp_add(sigrifft_perform, 2, out1, n);
+    dsp_add(sigrifft_perform, 2, out1, (t_int)n);
 }
 
 static void sigrifft_setup(void)
@@ -349,8 +349,8 @@ static void sigframp_dsp(t_sigframp *x, t_signal **sp)
         return;
     }
     dsp_add(sigframp_perform, 5, sp[0]->s_vec, sp[1]->s_vec,
-        sp[2]->s_vec, sp[3]->s_vec, n2);
-    dsp_add(sigsqrt_perform, 3, sp[3]->s_vec, sp[3]->s_vec, n2);
+        sp[2]->s_vec, sp[3]->s_vec, (t_int)n2);
+    dsp_add(sigsqrt_perform, 3, sp[3]->s_vec, sp[3]->s_vec, (t_int)n2);
 }
 
 static void sigframp_setup(void)
