@@ -342,7 +342,7 @@ int open_soundfile_via_fd(int fd, t_soundfile_info *p_info, long skipframes)
             nchannels = 1;
             bytespersamp = 2;
             samprate = 44100;
-                /* copy the first chunk header to beginnning of buffer. */
+                /* copy the first chunk header to beginning of buffer. */
             memcpy(buf.b_c, buf.b_c + headersize, sizeof(t_wavechunk));
             /* post("chunk %c %c %c %c",
                     ((t_wavechunk *)buf)->wc_id[0],
@@ -403,7 +403,7 @@ int open_soundfile_via_fd(int fd, t_soundfile_info *p_info, long skipframes)
             nchannels = 1;
             bytespersamp = 2;
             samprate = 44100;
-                /* copy the first chunk header to beginnning of buffer. */
+                /* copy the first chunk header to beginning of buffer. */
             memcpy(buf.b_c, buf.b_c + headersize, sizeof(t_datachunk));
                 /* read chunks in loop until we get to the data chunk */
             datachunk = &buf.b_datachunk;
@@ -517,7 +517,7 @@ static void soundfile_xferin_sample(int sfchannels, int nvecs, t_sample **vecs,
     int nchannels = (sfchannels < nvecs ? sfchannels : nvecs);
     int bytespersample = bytespersamp;
     int bytesperframe = bytespersample * sfchannels;
-    int framesread = itemsread, nframes = nitems;
+    long framesread = itemsread, nframes = nitems;
     for (i = 0, sp = buf; i < nchannels; i++, sp += bytespersample)
     {
         if (bytespersample == 2)
@@ -1726,6 +1726,9 @@ typedef struct _readsf
     pthread_cond_t x_requestcondition;
     pthread_cond_t x_answercondition;
     pthread_t x_childthread;
+#ifdef PDINSTANCE
+    t_pdinstance *x_pd_this; /* pointer to the owner pd instance */
+#endif
 } t_readsf;
 
 
@@ -1762,6 +1765,9 @@ static void readsf_fakewait(pthread_mutex_t *b)
 static void *readsf_child_main(void *zz)
 {
     t_readsf *x = zz;
+#ifdef PDINSTANCE
+    pd_this = x->x_pd_this;
+#endif
 #ifdef DEBUG_SOUNDFILE
     pute("1\n");
 #endif
@@ -2085,6 +2091,9 @@ static void *readsf_new(t_floatarg fnchannels, t_floatarg fbufsize)
     x->x_buf = buf;
     x->x_bufsize = bufsize;
     x->x_fifosize = x->x_fifohead = x->x_fifotail = x->x_requestcode = 0;
+#ifdef PDINSTANCE
+    x->x_pd_this = pd_this;
+#endif
     pthread_create(&x->x_childthread, 0, readsf_child_main, x);
     return (x);
 }
@@ -2320,6 +2329,9 @@ static t_class *writesf_class;
 static void *writesf_child_main(void *zz)
 {
     t_writesf *x = zz;
+#ifdef PDINSTANCE
+    pd_this = x->x_pd_this;
+#endif
 #ifdef DEBUG_SOUNDFILE
     pute("1\n");
 #endif
@@ -2413,8 +2425,7 @@ static void *writesf_child_main(void *zz)
                 pute("open failed\n");
                 pute(filename);
 #endif
-                x->x_requestcode = REQUEST_NOTHING;
-                continue;
+                goto bail;
             }
             /* check if another request has been made; if so, field it */
             if (x->x_requestcode != REQUEST_BUSY)
@@ -2484,7 +2495,7 @@ static void *writesf_child_main(void *zz)
                     pute("fileerror\n");
 #endif
                     x->x_fileerror = errno;
-                    break;
+                    goto bail;
                 }
                 else
                 {
@@ -2502,6 +2513,22 @@ static void *writesf_child_main(void *zz)
                     /* signal parent in case it's waiting for data */
                 sfread_cond_signal(&x->x_answercondition);
             }
+            continue;
+
+        bail:
+            if (x->x_requestcode == REQUEST_BUSY)
+                x->x_requestcode = REQUEST_NOTHING;
+                /* hit an error; close file if necessary,
+                set EOF and signal once more */
+            if (x->x_fd >= 0)
+            {
+                pthread_mutex_unlock(&x->x_mutex);
+                close (x->x_fd);
+                pthread_mutex_lock(&x->x_mutex);
+                x->x_eof = 1;
+                x->x_fd = -1;
+            }
+            sfread_cond_signal(&x->x_answercondition);
         }
         else if (x->x_requestcode == REQUEST_CLOSE ||
             x->x_requestcode == REQUEST_QUIT)
@@ -2586,6 +2613,9 @@ static void *writesf_new(t_floatarg fnchannels, t_floatarg fbufsize)
     x->x_buf = buf;
     x->x_bufsize = bufsize;
     x->x_fifosize = x->x_fifohead = x->x_fifotail = x->x_requestcode = 0;
+#ifdef PDINSTANCE
+    x->x_pd_this = pd_this;
+#endif
     pthread_create(&x->x_childthread, 0, writesf_child_main, x);
     return (x);
 }
@@ -2604,7 +2634,7 @@ static t_int *writesf_perform(t_int *w)
         roominfifo = x->x_fifotail - x->x_fifohead;
         if (roominfifo <= 0)
             roominfifo += x->x_fifosize;
-        while (roominfifo < wantbytes + 1)
+        while (!x->x_eof && roominfifo < wantbytes + 1)
         {
             fprintf(stderr, "writesf waiting for disk write..\n");
             fprintf(stderr, "(head %d, tail %d, room %d, want %d)\n",
@@ -2616,6 +2646,20 @@ static t_int *writesf_perform(t_int *w)
             if (roominfifo <= 0)
                 roominfifo += x->x_fifosize;
         }
+        if (x->x_eof)
+        {
+            if (x->x_fileerror)
+            {
+                pd_error(x, "dsp: %s: %s", x->x_filename,
+                    (x->x_fileerror == EIO ?
+                        "unknown or bad header format" :
+                            strerror(x->x_fileerror)));
+            }
+            x->x_state = STATE_IDLE;
+            sfread_cond_signal(&x->x_requestcondition);
+            pthread_mutex_unlock(&x->x_mutex);
+            return (w+2);
+        }
 
         soundfile_xferout_sample(sfchannels, x->x_outvec,
             (unsigned char *)(x->x_buf + x->x_fifohead), vecsize, 0,
