@@ -852,9 +852,9 @@ int garray_getfloatarray(t_garray *x, int *size, t_float **vec)
             patchname = x->x_glist->gl_owner->gl_name;
         else
             patchname = x->x_glist->gl_name;
-        error("An operation on the array '%s' in the patch '%s'",
+        error("an operation on the array '%s' in the patch '%s'",
               x->x_name->s_name, patchname->s_name);
-        error("failed since it uses garray_getfloatarray while running 64-bit!");
+        error("failed since it uses garray_getfloatarray while running 64-bit");
     }
     return (garray_getfloatwords(x, size, (t_word **)vec));
 }
@@ -1160,6 +1160,8 @@ void garray_resize_long(t_garray *x, long n)
     t_array *array = garray_getarray(x);
     if (n < 1)
         n = 1;
+    if (n == array->a_n)
+        return;
     garray_fittograph(x, (int)n, template_getfloat(
         template_findbyname(x->x_scalar->sc_template),
             gensym("style"), x->x_scalar->sc_vec, 1));
