@@ -313,6 +313,10 @@ static void rtext_senditup(t_rtext *x, int action, int *widthp, int *heightp,
     }
     escbuf = (tempbuf == smallbuf)?smallescbuf:t_getbytes(2 * outchars_b + 1);
     pdgui_strnescape(escbuf, 2 * outchars_b + 1, tempbuf, outchars_b);
+
+    if (action && !canvas->gl_havewindow)
+        action = 0;
+
     if (action == SEND_FIRST)
     {
         int lmargin = LMARGIN, tmargin = TMARGIN;
@@ -538,7 +542,7 @@ void rtext_key(t_rtext *x, int keynum, t_symbol *keysym)
         x->x_buf = resizebytes(x->x_buf, x->x_bufsize, newsize);
         x->x_bufsize = newsize;
 
-/* at Guenter's suggestion, use 'n>31' to test wither a character might
+/* at Guenter's suggestion, use 'n>31' to test whether a character might
 be printable in whatever 8-bit character set we find ourselves. */
 
 /*-- moo:
