@@ -72,6 +72,10 @@ static void vslider_draw_new(t_vslider *x, t_glist *glist)
              xpos, ypos - tmargin,
              xpos + iow, ypos - tmargin - IEMGUI_ZOOM(x) + ioh,
              x, 0);
+    sys_vgui(".x%lx.c create line %d %d %d %d -width %d -fill #%06x -tags %lxKNOB\n",
+             canvas, xpos + IEMGUI_ZOOM(x), r,
+             xpos + x->x_gui.x_w - IEMGUI_ZOOM(x), r,
+             1 + 2 * IEMGUI_ZOOM(x), x->x_gui.x_fcol, x);
     sys_vgui(".x%lx.c create text %d %d -text {%s} -anchor w \
              -font {{%s} -%d %s} -fill #%06x -tags [list %lxLABEL label text]\n",
              canvas, xpos + x->x_gui.x_ldx * IEMGUI_ZOOM(x),
@@ -79,10 +83,6 @@ static void vslider_draw_new(t_vslider *x, t_glist *glist)
              (strcmp(x->x_gui.x_lab->s_name, "empty") ? x->x_gui.x_lab->s_name : ""),
              x->x_gui.x_font, x->x_gui.x_fontsize * IEMGUI_ZOOM(x), sys_fontweight,
              x->x_gui.x_lcol, x);
-    sys_vgui(".x%lx.c create line %d %d %d %d -width %d -fill #%06x -tags %lxKNOB\n",
-             canvas, xpos + IEMGUI_ZOOM(x), r,
-             xpos + x->x_gui.x_w - IEMGUI_ZOOM(x), r,
-             1 + 2 * IEMGUI_ZOOM(x), x->x_gui.x_fcol, x);
 }
 
 static void vslider_draw_move(t_vslider *x, t_glist *glist)
@@ -108,12 +108,12 @@ static void vslider_draw_move(t_vslider *x, t_glist *glist)
              canvas, x, 0,
              xpos, ypos - tmargin,
              xpos + iow, ypos - tmargin - IEMGUI_ZOOM(x) + ioh);
-    sys_vgui(".x%lx.c coords %lxLABEL %d %d\n",
-             canvas, x, xpos+x->x_gui.x_ldx * IEMGUI_ZOOM(x),
-             ypos+x->x_gui.x_ldy * IEMGUI_ZOOM(x));
     sys_vgui(".x%lx.c coords %lxKNOB %d %d %d %d\n",
              canvas, x, xpos + IEMGUI_ZOOM(x), r,
              xpos + x->x_gui.x_w - IEMGUI_ZOOM(x), r);
+    sys_vgui(".x%lx.c coords %lxLABEL %d %d\n",
+             canvas, x, xpos+x->x_gui.x_ldx * IEMGUI_ZOOM(x),
+             ypos+x->x_gui.x_ldy * IEMGUI_ZOOM(x));
 }
 
 static void vslider_draw_erase(t_vslider* x, t_glist* glist)
@@ -158,8 +158,9 @@ static void vslider_draw_io(t_vslider* x, t_glist* glist, int old_snd_rcv_flags)
              xpos, ypos + x->x_gui.x_h + bmargin + IEMGUI_ZOOM(x) - ioh,
              xpos + iow, ypos + x->x_gui.x_h + bmargin,
              x, 0);
-        /* keep knob above outlet */
+        /* keep these above outlet */
         sys_vgui(".x%lx.c raise %lxKNOB %lxOUT%d\n", canvas, x, x, 0);
+        sys_vgui(".x%lx.c raise %lxLABEL %lxKNOB\n", canvas, x, x);
     }
     if(!(old_snd_rcv_flags & IEM_GUI_OLD_SND_FLAG) && x->x_gui.x_fsf.x_snd_able)
         sys_vgui(".x%lx.c delete %lxOUT%d\n", canvas, x, 0);
@@ -170,8 +171,9 @@ static void vslider_draw_io(t_vslider* x, t_glist* glist, int old_snd_rcv_flags)
              xpos, ypos - tmargin,
              xpos + iow, ypos - tmargin - IEMGUI_ZOOM(x) + ioh,
              x, 0);
-        /* keep label and knob above inlet */
+        /* keep these above inlet */
         sys_vgui(".x%lx.c raise %lxKNOB %lxIN%d\n", canvas, x, x, 0);
+        sys_vgui(".x%lx.c raise %lxLABEL %lxKNOB\n", canvas, x, x);
     }
     if(!(old_snd_rcv_flags & IEM_GUI_OLD_RCV_FLAG) && x->x_gui.x_fsf.x_rcv_able)
         sys_vgui(".x%lx.c delete %lxIN%d\n", canvas, x, 0);
