@@ -21,7 +21,6 @@
 #define MAX_CLIENTS 100
 #define MAX_JACK_PORTS 128  /* higher values seem to give bad xrun problems */
 #define BUF_JACK 4096
-#define JACK_OUT_MAX  64
 
 static jack_nframes_t jack_out_max;
 static jack_nframes_t jack_filled = 0;
@@ -46,9 +45,8 @@ static int pollprocess(jack_nframes_t nframes, void *arg)
     jack_default_audio_sample_t *out, *in;
 
     pthread_mutex_lock(&jack_mutex);
-    if (nframes > JACK_OUT_MAX) jack_out_max = nframes;
-    else jack_out_max = JACK_OUT_MAX;
-    if (jack_filled >= nframes)
+    jack_out_max = nframes;
+    if (nframes >= DEFDACBLKSIZE && jack_filled >= nframes)
     {
         if (jack_filled != nframes)
             fprintf(stderr,"Partial read\n");
@@ -58,13 +56,13 @@ static int pollprocess(jack_nframes_t nframes, void *arg)
         {
             for (j = 0; j < STUFF->st_outchannels;  j++)
             {
-                if (out = jack_port_get_buffer(output_port[j], nframes))
+                if ((out = jack_port_get_buffer(output_port[j], nframes)))
                     memcpy(out, jack_outbuf + (j * BUF_JACK),
                         sizeof (jack_default_audio_sample_t) * nframes);
             }
             for (j = 0; j < STUFF->st_inchannels; j++)
             {
-                if (in = jack_port_get_buffer(input_port[j], nframes))
+                if ((in = jack_port_get_buffer(input_port[j], nframes)))
                     memcpy(jack_inbuf + (j * BUF_JACK), in,
                         sizeof (jack_default_audio_sample_t) * nframes);
             }
@@ -75,7 +73,7 @@ static int pollprocess(jack_nframes_t nframes, void *arg)
             t_sample*data;
             for (j = 0; j < STUFF->st_outchannels;  j++)
             {
-                if (out = jack_port_get_buffer (output_port[j], nframes))
+                if ((out = jack_port_get_buffer(output_port[j], nframes)))
                 {
                     data = jack_outbuf + (j * BUF_JACK);
                     for (frame=0; frame<nframes; frame++)
@@ -84,7 +82,7 @@ static int pollprocess(jack_nframes_t nframes, void *arg)
             }
             for (j = 0; j < STUFF->st_inchannels; j++)
             {
-                if (in = jack_port_get_buffer( input_port[j], nframes))
+                if ((in = jack_port_get_buffer(input_port[j], nframes)))
                 {
                     data = jack_inbuf + (j * BUF_JACK);
                     for (frame=0; frame<nframes; frame++)
@@ -96,10 +94,17 @@ static int pollprocess(jack_nframes_t nframes, void *arg)
     }
     else
     {           /* PD could not keep up ! */
+        if (nframes < DEFDACBLKSIZE)
+        {
+            static int firsttime = 1;
+            if(firsttime)
+                fprintf(stderr,"jack: nframes %d smaller than blocksize %d: NO SOUND!\n", nframes, DEFDACBLKSIZE);
+            firsttime = 0;
+        }
         if (jack_started) jack_dio_error = 1;
         for (j = 0; j < outport_count;  j++)
         {
-            if (out = jack_port_get_buffer (output_port[j], nframes))
+            if ((out = jack_port_get_buffer(output_port[j], nframes)))
                 memset(out, 0, sizeof (jack_default_audio_sample_t) * nframes);
             memset(jack_outbuf + j * BUF_JACK, 0, BUF_JACK * sizeof(t_sample));
         }
