@@ -16,26 +16,19 @@ standard output. */
 #include <stdio.h>
 #include <errno.h>
 #include <stdlib.h>
-#ifdef _WIN32
-#include <winsock.h>
-#else
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <netinet/tcp.h>
-#include <netdb.h>
-#include <stdio.h>
+#ifndef _WIN32
 #include <unistd.h>
-#include <sys/select.h>
-#define SOCKET_ERROR -1
 #endif
 
+#include "s_net.h"
+
 typedef struct _fdpoll
 {
     int fdp_fd;
-    char *fdp_outbuf;/*output message buffer*/
-    int fdp_outlen;     /*length of output message*/
-    int fdp_discard;/*buffer overflow: output message is incomplete, discard it*/
-    int fdp_gotsemi;/*last char from input was a semicolon*/
+    char *fdp_outbuf; /*output message buffer*/
+    int fdp_outlen;   /*length of output message*/
+    int fdp_discard;  /*buffer overflow: output message is incomplete, discard it*/
+    int fdp_gotsemi;  /*last char from input was a semicolon*/
 } t_fdpoll;
 
 static int nfdpoll;
@@ -45,22 +38,22 @@ static int sockfd;
 static int protocol;
 
 static void sockerror(char *s);
-static void x_closesocket(int fd);
 static void dopoll(void);
+static void sockerror(char *s);
+
+/* print addrinfo lists for debugging */
+/* #define PRINT_ADDRINFO */
+
 #define BUFSIZE 4096
 
 int main(int argc, char **argv)
 {
-    int portno;
-    struct sockaddr_in server = {0};
-    int nretry = 10;
-#ifdef _WIN32
-    short version = MAKEWORD(2, 0);
-    WSADATA nobby;
-#endif
+    int status, portno, multicast = 0;
+    char *hostname = NULL;
+    struct addrinfo *ailist = NULL, *ai;
     if (argc < 2 || sscanf(argv[1], "%d", &portno) < 1 || portno <= 0)
         goto usage;
-    if (argc >= 3)
+    if (argc > 2)
     {
         if (!strcmp(argv[2], "tcp"))
             protocol = SOCK_STREAM;
@@ -68,47 +61,163 @@ int main(int argc, char **argv)
             protocol = SOCK_DGRAM;
         else goto usage;
     }
-    else protocol = SOCK_STREAM;
-#ifdef _WIN32
-    if (WSAStartup(version, &nobby)) sockerror("WSAstartup");
+    else
+        protocol = SOCK_STREAM; /* default */
+    if (argc > 3)
+        hostname = argv[3];
+    if (socket_init())
+    {
+        sockerror("socket_init()");
+        exit(EXIT_FAILURE);
+    }
+    status = addrinfo_get_list(&ailist, hostname, portno, protocol);
+    if (status != 0)
+    {
+        fprintf(stderr, "bad host or port? %s (%d)\n",
+            gai_strerror(status), status);
+        exit(EXIT_FAILURE);
+    }
+    if (hostname)
+    {
+        /* If we specify a hostname or IP address we can only listen to a single adapter.
+         * For host names, we prefer IPv4 for now. LATER we might create several sockets */
+        addrinfo_sort_list(&ailist, addrinfo_ipv4_first);
+    }
+    else
+    {
+        /* For the "any" address we want to prefer IPv6, so we can create a dual stack socket */
+        addrinfo_sort_list(&ailist, addrinfo_ipv6_first);
+    }
+#ifdef PRINT_ADDRINFO
+    addrinfo_print_list(ailist);
 #endif
-    sockfd = socket(AF_INET, protocol, 0);
+    /* try each addr until we find one that works */
+    for (ai = ailist; ai != NULL; ai = ai->ai_next)
+    {
+        sockfd = socket(ai->ai_family, ai->ai_socktype, ai->ai_protocol);
+        if (sockfd < 0)
+            continue;
+    #if 1
+        /* ask OS to allow another process to reopen this port after we close it */
+        if (socket_set_boolopt(sockfd, SOL_SOCKET, SO_REUSEADDR, 1) < 0)
+            fprintf(stderr, "setsockopt (SO_REUSEADDR) failed\n");
+    #endif
+        if (protocol == SOCK_STREAM)
+        {
+            /* stream (TCP) sockets are set NODELAY */
+            if (socket_set_boolopt(sockfd, IPPROTO_TCP, TCP_NODELAY, 1) < 0)
+                fprintf(stderr, "setsockopt (TCP_NODELAY) failed\n");
+        }
+        else if (protocol == SOCK_DGRAM && ai->ai_family == AF_INET)
+        {
+            /* enable IPv4 UDP broadcasting */
+            if (socket_set_boolopt(sockfd, SOL_SOCKET, SO_BROADCAST, 1) < 0)
+                fprintf(stderr, "setsockopt (SO_BROADCAST) failed\n");
+        }
+        /* if this is an IPv6 address, also listen to IPv4 adapters
+           (if not supported, fall back to IPv4) */
+        if (ai->ai_family == AF_INET6 &&
+                socket_set_boolopt(sockfd, IPPROTO_IPV6, IPV6_V6ONLY, 0) < 0)
+        {
+            /* fprintf(stderr, "setsockopt (IPV6_V6ONLY) failed\n"); */
+            socket_close(sockfd);
+            sockfd = -1;
+            continue;
+        }
+        multicast = sockaddr_is_multicast(ai->ai_addr);
+#if 1
+        if (multicast)
+        {
+            /* binding to the multicast address doesn't work on Windows and on Linux
+               it doesn't seem to work for IPv6 multicast addresses, so we bind to
+               the "any" address instead */
+            struct addrinfo *any;
+            int status = addrinfo_get_list(&any,
+                (ai->ai_family == AF_INET6) ? "::" : "0.0.0.0", portno, protocol);
+            if (status != 0)
+            {
+                fprintf(stderr,
+                    "getting \"any\" address for multicast failed %s (%d)\n",
+                    gai_strerror(status), status);
+                socket_close(sockfd);
+                return EXIT_FAILURE;
+            }
+            /* name the socket */
+            status = bind(sockfd, any->ai_addr, any->ai_addrlen);
+            freeaddrinfo(any);
+            if (status < 0)
+            {
+                socket_close(sockfd);
+                sockfd = -1;
+                continue;
+            }
+        }
+        else
+#endif
+        {
+            /* name the socket */
+            if (bind(sockfd, ai->ai_addr, ai->ai_addrlen) < 0)
+            {
+                socket_close(sockfd);
+                sockfd = -1;
+                continue;
+            }
+        }
+        /* join multicast group */
+        if (multicast && socket_join_multicast_group(sockfd, ai->ai_addr) < 0)
+        {
+            int err = socket_errno();
+            char buf[256];
+            socket_strerror(err, buf, sizeof(buf));
+            fprintf(stderr,
+                "joining multicast group %s failed: %s (%d)\n",
+                hostname, buf, err);
+        }
+        /* this addr worked */
+        if (hostname)
+        {
+            char hostbuf[256];
+            sockaddr_get_addrstr(ai->ai_addr,
+                hostbuf, sizeof(hostbuf));
+            fprintf(stderr, "listening on %s %d%s\n", hostbuf, portno,
+                (multicast ? " (multicast)" : ""));
+        }
+        else
+            fprintf(stderr, "listening on %d\n", portno);
+        break;
+    }
+    freeaddrinfo(ailist);
+
+    /* confirm that socket/bind worked */
     if (sockfd < 0)
     {
-        sockerror("socket()");
-        exit(1);
+        int err = socket_errno();
+        char buf[256];
+        socket_strerror(err, buf, sizeof(buf));
+        fprintf(stderr, "listen failed: %s (%d)\n", buf, err);
+        exit(EXIT_FAILURE);
     }
+
     maxfd = sockfd + 1;
-    server.sin_family = AF_INET;
-    server.sin_addr.s_addr = INADDR_ANY;
 
-        /* assign client port number */
-    server.sin_port = htons((unsigned short)portno);
-
-        /* name the socket */
-    if (bind(sockfd, (struct sockaddr *)&server, sizeof(server)) < 0)
-    {
-        sockerror("bind");
-        x_closesocket(sockfd);
-        return (0);
-    }
-    if (protocol == SOCK_STREAM)
+    if (protocol == SOCK_STREAM) /* streaming protocol */
     {
         if (listen(sockfd, 5) < 0)
         {
             sockerror("listen");
-            x_closesocket(sockfd);
-            exit(1);
+            socket_close(sockfd);
+            exit(EXIT_FAILURE);
         }
     }
-        /* now loop forever selecting on sockets */
+
+    /* now loop forever selecting on sockets */
     while (1)
         dopoll();
 
 usage:
-    fprintf(stderr, "usage: pdreceive <portnumber> [udp|tcp]\n");
+    fprintf(stderr, "usage: pdreceive <portnumber> [udp|tcp] [host]\n");
     fprintf(stderr, "(default is tcp)\n");
-    exit(1);
+    exit(EXIT_FAILURE);
 }
 
 static void addport(int fd)
@@ -121,7 +230,7 @@ static void addport(int fd)
     {
         free(fdpoll);
         fprintf(stderr, "out of memory!");
-        exit(1);
+        exit(EXIT_FAILURE);
     }
     fdpoll = fdtmp;
     fp = fdpoll + nfdpoll;
@@ -132,7 +241,7 @@ static void addport(int fd)
     if (!(fp->fdp_outbuf = (char*) malloc(BUFSIZE)))
     {
         fprintf(stderr, "out of memory");
-        exit(1);
+        exit(EXIT_FAILURE);
     }
     printf("number_connected %d;\n", nfdpoll);
 }
@@ -146,7 +255,7 @@ static void rmport(t_fdpoll *x)
     {
         if (fp == x)
         {
-            x_closesocket(fp->fdp_fd);
+            socket_close(fp->fdp_fd);
             free(fp->fdp_outbuf);
             while (i--)
             {
@@ -181,7 +290,7 @@ static void makeoutput(char *buf, int len)
     if (write(1, buf, len) < len)
     {
         perror("write");
-        exit(1);
+        exit(EXIT_FAILURE);
     }
 #endif
 }
@@ -193,8 +302,8 @@ static void udpread(void)
     if (ret < 0)
     {
         sockerror("recv (udp)");
-        x_closesocket(sockfd);
-        exit(1);
+        socket_close(sockfd);
+        exit(EXIT_FAILURE);
     }
     else if (ret > 0)
         makeoutput(buf, ret);
@@ -270,7 +379,7 @@ static void dopoll(void)
     if (select(maxfd+1, &readset, &writeset, &exceptset, 0) < 0)
     {
         perror("select");
-        exit(1);
+        exit(EXIT_FAILURE);
     }
     if (protocol == SOCK_STREAM)
     {
@@ -288,27 +397,10 @@ static void dopoll(void)
 }
 
 
-static void sockerror(char *s)
+void sockerror(char *s)
 {
-#ifdef _WIN32
-    int err = WSAGetLastError();
-    if (err == 10054) return;
-    else if (err == 10044)
-    {
-        fprintf(stderr,
-            "Warning: you might not have TCP/IP \"networking\" turned on\n");
-    }
-#else
-    int err = errno;
-#endif
-    fprintf(stderr, "%s: %s (%d)\n", s, strerror(err), err);
-}
-
-static void x_closesocket(int fd)
-{
-#ifdef _WIN32
-    closesocket(fd);
-#else
-    close(fd);
-#endif
+    char buf[256];
+    int err = socket_errno();
+    socket_strerror(err, buf, sizeof(buf));
+    fprintf(stderr, "%s: %s (%d)\n", s, buf, err);
 }
