@@ -106,7 +106,7 @@ static void array_define_yrange(t_glist *x, t_floatarg ylo, t_floatarg yhi)
             "ffff", 0., yhi, (double)(n == 1 ? n : n-1), ylo);
         vmess(&x->gl_list->g_pd, gensym("xlabel"),
             "fff", ylo + glist_pixelstoy(gl, 2) - glist_pixelstoy(gl, 0),
-                0., (float)(n-1));
+                0., (t_float)(n-1));
         vmess(&x->gl_list->g_pd, gensym("ylabel"),
             "fff", glist_pixelstox(gl, 0) - glist_pixelstox(gl, 5), ylo, yhi);
     }
@@ -116,11 +116,11 @@ static void array_define_yrange(t_glist *x, t_floatarg ylo, t_floatarg yhi)
 static void *array_define_new(t_symbol *s, int argc, t_atom *argv)
 {
     t_symbol *arrayname = &s_;
-    float arraysize = 100;
+    t_float arraysize = 100;
     t_glist *x;
     int keep = 0;
-    float ylo = -1, yhi = 1;
-    float xpix = 500, ypix = 300;
+    t_float ylo = -1, yhi = 1;
+    t_float xpix = 500, ypix = 300;
     while (argc && argv->a_type == A_SYMBOL &&
         *argv->a_w.w_symbol->s_name == '-')
     {
@@ -186,7 +186,7 @@ void array_define_save(t_gobj *z, t_binbuf *bb)
     t_glist *x = (t_glist *)z;
     t_glist *gl = (x->gl_list ? pd_checkglist(&x->gl_list->g_pd) : 0);
     binbuf_addv(bb, "ssff", &s__X, gensym("obj"),
-        (float)x->gl_obj.te_xpix, (float)x->gl_obj.te_ypix);
+        (t_float)x->gl_obj.te_xpix, (t_float)x->gl_obj.te_ypix);
     binbuf_addbinbuf(bb, x->gl_obj.ob_binbuf);
     binbuf_addsemi(bb);
 
