@@ -192,12 +192,15 @@ typedef struct _openpanel
 {
     t_object x_obj;
     t_symbol *x_s;
+    int x_mode; /* 0: file, 1: folder, 2: multiple files */
 } t_openpanel;
 
-static void *openpanel_new(void)
+static void *openpanel_new(t_floatarg mode)
 {
     char buf[50];
+    int m = (int)mode;
     t_openpanel *x = (t_openpanel *)pd_new(openpanel_class);
+    x->x_mode = (mode < 0 || mode > 2) ? 0 : mode;
     sprintf(buf, "d%lx", (t_int)x);
     x->x_s = gensym(buf);
     pd_bind(&x->x_obj.ob_pd, x->x_s);
@@ -208,7 +211,8 @@ static void *openpanel_new(void)
 static void openpanel_symbol(t_openpanel *x, t_symbol *s)
 {
     const char *path = (s && s->s_name) ? s->s_name : "\"\"";
-    sys_vgui("pdtk_openpanel {%s} {%s}\n", x->x_s->s_name, path);
+    sys_vgui("pdtk_openpanel {%s} {%s} %d\n",
+        x->x_s->s_name, path, x->x_mode);
 }
 
 static void openpanel_bang(t_openpanel *x)
@@ -216,12 +220,19 @@ static void openpanel_bang(t_openpanel *x)
     openpanel_symbol(x, &s_);
 }
 
-static void openpanel_callback(t_openpanel *x, t_symbol *s)
+static void openpanel_callback(t_openpanel *x, t_symbol *s, int argc, t_atom *argv)
 {
-    outlet_symbol(x->x_obj.ob_outlet, s);
+    if (x->x_mode != 2) /* single file or folder */
+    {
+        if (argc == 1 && argv->a_type == A_SYMBOL)
+            outlet_symbol(x->x_obj.ob_outlet, argv->a_w.w_symbol);
+        else
+            bug("openpanel_callback");
+    }
+    else /* list of files */
+        outlet_list(x->x_obj.ob_outlet, s, argc, argv);
 }
 
-
 static void openpanel_free(t_openpanel *x)
 {
     pd_unbind(&x->x_obj.ob_pd, x->x_s);
@@ -231,11 +242,11 @@ static void openpanel_setup(void)
 {
     openpanel_class = class_new(gensym("openpanel"),
         (t_newmethod)openpanel_new, (t_method)openpanel_free,
-        sizeof(t_openpanel), 0, 0);
+        sizeof(t_openpanel), 0, A_DEFFLOAT, 0);
     class_addbang(openpanel_class, openpanel_bang);
     class_addsymbol(openpanel_class, openpanel_symbol);
     class_addmethod(openpanel_class, (t_method)openpanel_callback,
-        gensym("callback"), A_SYMBOL, 0);
+        gensym("callback"), A_GIMME, 0);
 }
 
 /* -------------------------- savepanel ------------------------------ */
