@@ -134,6 +134,7 @@ proc ::dialog_audio::pdtk_audio_dialog {mytoplevel \
     set audio_blocksize $blocksize
 
     toplevel $mytoplevel -class DialogWindow
+    wm withdraw $mytoplevel
     wm title $mytoplevel [_ "Audio Settings"]
     wm group $mytoplevel .
     wm resizable $mytoplevel 0 0
@@ -142,8 +143,6 @@ proc ::dialog_audio::pdtk_audio_dialog {mytoplevel \
     $mytoplevel configure -menu $::dialog_menubar
     $mytoplevel configure -padx 10 -pady 5
     ::pd_bindings::dialog_bindings $mytoplevel "audio"
-    # not all Tcl/Tk versions or platforms support -topmost, so catch the error
-    catch {wm attributes $mytoplevel -topmost 1}
 
     # settings
     labelframe $mytoplevel.settings -text [_ "Settings"] -padx 5 -pady 5 -borderwidth 1
@@ -160,17 +159,29 @@ proc ::dialog_audio::pdtk_audio_dialog {mytoplevel \
 
     frame $mytoplevel.settings.bsc
     pack $mytoplevel.settings.bsc -side top -fill x
+    button $mytoplevel.settings.bsc.rate1 -text [_ "48k"] \
+        -command "set audio_sr 48000"
+    button $mytoplevel.settings.bsc.rate2 -text [_ "44.1k"] \
+        -command "set audio_sr 44100"
+    button $mytoplevel.settings.bsc.rate3 -text [_ "96k"] \
+        -command "set audio_sr 96000"
+    pack $mytoplevel.settings.bsc.rate1 \
+        $mytoplevel.settings.bsc.rate2 \
+        $mytoplevel.settings.bsc.rate3 \
+         -side left
     label $mytoplevel.settings.bsc.bs_label -text [_ "Block size:"]
     set blocksizes {64 128 256 512 1024 2048}
     set bsmenu \
         [eval tk_optionMenu $mytoplevel.settings.bsc.bs_popup audio_blocksize $blocksizes]
 
-    pack $mytoplevel.settings.bsc.bs_label -side left -padx {0 10}
-    pack $mytoplevel.settings.bsc.bs_popup -side left
+    pack $mytoplevel.settings.bsc.bs_popup -side right
+    pack $mytoplevel.settings.bsc.bs_label -side right -padx {0 10}
     if {$audio_callback >= 0} {
-        checkbutton $mytoplevel.settings.bsc.c_button -variable audio_callback \
-            -text [_ "Use callbacks"] -anchor e
-        pack $mytoplevel.settings.bsc.c_button -side right
+        frame $mytoplevel.settings.callback
+        pack $mytoplevel.settings.callback -side bottom -fill x
+        checkbutton $mytoplevel.settings.callback.c_button -variable audio_callback \
+            -text [_ "Use callbacks"]
+        pack $mytoplevel.settings.callback.c_button -side right
     }
 
     # input devices
@@ -343,11 +354,9 @@ proc ::dialog_audio::pdtk_audio_dialog {mytoplevel \
     button $mytoplevel.buttonframe.cancel -text [_ "Cancel"] \
         -command "::dialog_audio::cancel $mytoplevel"
     pack $mytoplevel.buttonframe.cancel -side left -expand 1 -fill x -padx 15 -ipadx 10
-    if {$::windowingsystem ne "aqua"} {
-        button $mytoplevel.buttonframe.apply -text [_ "Apply"] \
-            -command "::dialog_audio::apply $mytoplevel"
-        pack $mytoplevel.buttonframe.apply -side left -expand 1 -fill x -padx 15 -ipadx 10
-    }
+    button $mytoplevel.buttonframe.apply -text [_ "Apply"] \
+        -command "::dialog_audio::apply $mytoplevel"
+    pack $mytoplevel.buttonframe.apply -side left -expand 1 -fill x -padx 15 -ipadx 10
     button $mytoplevel.buttonframe.ok -text [_ "OK"] \
         -command "::dialog_audio::ok $mytoplevel" -default active
     pack $mytoplevel.buttonframe.ok -side left -expand 1 -fill x -padx 15 -ipadx 10
@@ -402,9 +411,7 @@ proc ::dialog_audio::pdtk_audio_dialog {mytoplevel \
     # set min size based on widget sizing & pos over pdwindow
     wm minsize $mytoplevel [winfo reqwidth $mytoplevel] [winfo reqheight $mytoplevel]
     position_over_window $mytoplevel .pdwindow
-
-    # wait a little for creation, then raise so it's on top
-    after 100 raise "$mytoplevel"
+    raise $mytoplevel
 }
 
 # for focus handling on OSX
